/* some functions to do lowlevel access to cdrom drives (currently using 
 * cdda2wav as direct cdrom ioctl()s did not do exactly what i wanted them
 * to do ;-) */
    
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <linux/cdrom.h>

#include "piping.h"
#include "cdromlow.h"
#include "preferences.h"
#include "varman.h"

/* reimplemented using direct cdrom access:
 * does the current cd have one or more data tracks ? */
int cdromlow_hasdatatrack(char *device)
{
	int cdrom;
	struct cdrom_tochdr header;
	struct cdrom_tocentry entry;
	int x;
	int hasdatatrack=0;	
	
	cdrom=open(device,O_RDONLY|O_NONBLOCK);
	ioctl(cdrom,CDROMREADTOCHDR,&header);	
	for (x=header.cdth_trk0;x<=header.cdth_trk1;x++)
	  {		  
		  entry.cdte_track=x;
		  entry.cdte_format=CDROM_LBA;
		  ioctl(cdrom,CDROMREADTOCENTRY,&entry);
		  if (entry.cdte_ctrl&CDROM_DATA_TRACK)
		      hasdatatrack=1;
	  }
	;
	close(cdrom);
	return hasdatatrack;
}
;

/* return number of tracks on cd,reimplemented using ioctl()s */
int cdromlow_tracks(char *device)
{
	int cdrom;
	struct cdrom_tochdr header;
	int tracknum;
	
	cdrom=open(device,O_RDONLY|O_NONBLOCK);
	if (ioctl(cdrom,CDROMREADTOCHDR,&header)!=-1)
	    tracknum=1+header.cdth_trk1-header.cdth_trk0;	
	else
	    tracknum=0;
	close(cdrom);
	return (tracknum);	
}
;

/* return tracksize for audiotrack n,reimplemented using ioctl()s */
int cdromlow_tracksize(char *device,int num)
{
	int cdrom;
	struct cdrom_tocentry track,tracknext;
	
	cdrom=open(device,O_RDONLY|O_NONBLOCK);
	
	track.cdte_track=num;
	track.cdte_format=CDROM_LBA;
	ioctl(cdrom,CDROMREADTOCENTRY,&track);
	
	if (num==cdromlow_tracks(device))
	    tracknext.cdte_track=CDROM_LEADOUT;
	    else
	    tracknext.cdte_track=num+1;
	tracknext.cdte_format=CDROM_LBA;
	ioctl(cdrom,CDROMREADTOCENTRY,&tracknext);
		
	close(cdrom);
	
	return ((tracknext.cdte_addr.lba-track.cdte_addr.lba)*
		CD_FRAMESIZE_RAW);
}
;

int cdromlow_detectdatatracksize(int fd,int min,int max)
{
	int mid;
	int valid;
	char buf;
	
	int tracksize;
	
	if (max-min>1)
	  {
		  mid=(max+min)/2;
		  lseek(fd,mid,SEEK_SET);
		  valid=read(fd,&buf,1);
		  if (valid>0)
		      tracksize=cdromlow_detectdatatracksize(fd,mid,max);
		  else
		      tracksize=cdromlow_detectdatatracksize(fd,min,mid);
	  }
	else
	  {
		  lseek(fd,max,SEEK_SET);
		  if (read(fd,&buf,1)>0)
		      tracksize=max;
		  else
		      tracksize=min;
	  }
	;
	return tracksize;
}
;

int cdromlow_datatracksize(char *device)
{
	int fd;
	int ts;
	
	fd=open(device,O_RDONLY);
	
	/* add one as seek starts counting at zero */
	ts=cdromlow_detectdatatracksize(fd,0,MAXDATATRACKSIZE)+1;
	
	close(fd);
	return ts;
}
;
	
/* return cddb number of cdrom */
int cdromlow_cddbnumber(char *device)
{
	int cdrom;
	struct cdrom_tochdr header;
	struct cdrom_tocentry entry;
	int x,secs;	
	int cddbnum=0;
	
	cdrom=open(device,O_RDONLY|O_NONBLOCK);
	if (ioctl(cdrom,CDROMREADTOCHDR,&header)!=-1)
	  {		  	
		  entry.cdte_track=CDROM_LEADOUT;
		  entry.cdte_format=CDROM_LBA;
		  ioctl(cdrom,CDROMREADTOCENTRY,&entry);
		  secs=(entry.cdte_addr.lba/75);
		  
		  for (x=header.cdth_trk0;x<=header.cdth_trk1;x++)
		    {		  
			    entry.cdte_track=x;
			    entry.cdte_format=CDROM_LBA;
			    ioctl(cdrom,CDROMREADTOCENTRY,&entry);
			    cddbnum+=helpings_dsum((entry.cdte_addr.lba/75)+2);
			    if (x==header.cdth_trk0) secs-=(entry.cdte_addr.lba/75);
		    }
		  ;
		  
		  cddbnum<<=24;
		  cddbnum|=(secs<<8);
		  cddbnum|=(1+header.cdth_trk1-header.cdth_trk0);
	  }
	else
	    cddbnum=0;
	close(cdrom);
	return cddbnum;
}
;
