/* implements cdrom drives as source for track streams */

#include <gtk/gtk.h>
#include <string.h>

#include "main.h"
#include "cdsource.h"
#include "selectlist.h"
#include "datatrack.h"
#include "audiotrack.h"
#include "streams.h"
#include "preferences.h"
#include "varman.h"
#include "varmanwidgets.h"
#include "tracks.h"
#include "cddrives.h"

cdsource_cdinfo *cdsource_cdinfo_create(cddrives_cdinfo *cddinfo)
{
	cdsource_cdinfo *info;
	
	info=(cdsource_cdinfo*)malloc(sizeof(cdsource_cdinfo));
	
	info->driveinfo=cddinfo;
	info->widget=NULL;
	info->listinfo=NULL;

	return info;
}
;

void cdsource_tracklist_create(cdsource_cdinfo*info) /* create the tracklist widget for this drive */
{
	char *track[2];
	char sizebuf[20];
	GtkWidget *column;
	int count;
	
	if (info->widget==NULL) // not yet displaying,one learns of his mistakes ;-)
	  {
		  info->widget=gtk_scrolled_window_new(NULL,NULL);
		  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(info->widget),
						 GTK_POLICY_AUTOMATIC,
						 GTK_POLICY_AUTOMATIC);		  
		  
		  info->listinfo=selectlist_info_create("stream:",
							0,
							NULL,
							1,0,NULL,NULL,
							NULL);
		  column=selectlist_create(info->listinfo,2);
		  
		  gtk_clist_set_selection_mode(GTK_CLIST(column),
					       GTK_SELECTION_EXTENDED);
		  gtk_clist_set_column_justification(GTK_CLIST(column),
						     0,
						     GTK_JUSTIFY_LEFT);
		  gtk_clist_set_column_width(GTK_CLIST(column),0,196);
		  gtk_clist_set_column_justification(GTK_CLIST(column),
						     1,
						     GTK_JUSTIFY_RIGHT);
		  gtk_clist_set_column_width(GTK_CLIST(column),1,96);
		  
#ifdef DEBUG
		  printf ("displaying %i tracks...\n",info->driveinfo->tracks);
#endif
		  for (count=0;count<info->driveinfo->tracks;count++)
		    {
			    streams_register(info->driveinfo->track[count]);
			    
			    track[0]=(char*)&info->driveinfo->track[count]->name;
			    sprintf((char*)&sizebuf,"%d",tracks_tracksize(info->driveinfo->track[count]));
			    track[1]=(char*)&sizebuf;
		  
			    gtk_clist_append(GTK_CLIST(column),track);
		    }
		  ;
		  
		  gtk_widget_show(column);
		  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(info->widget),
							 column);
	  };
}
;

void cdsource_tracklist_destroy(cdsource_cdinfo*info) // destroy a tracklist
{
	int x;
	char *trackentry;
	
	if (info->widget!=NULL)
	  {
		  /* unregister the streams for this tracklist */
		  for (x=0;trackentry!=NULL;x++)
		    {
			    trackentry=NULL;
			    gtk_clist_get_text(GTK_CLIST(info->listinfo->self),
					       x,0,
					       &trackentry);
			    if (trackentry!=NULL)
			      {				      
				streams_unregisterbyname(trackentry);
			      }
			    ;
		    }
		  ;		  
		  gtk_widget_destroy(info->widget);
		  free(info->listinfo);
		  info->listinfo=NULL;
		  info->widget=NULL;
	  }
	;
}
;

GtkTree *cdsource_cdtree(GtkSignalFunc selectcd,
			 GtkSignalFunc unselectcd)
{
	GtkTree *tree;
	GtkWidget *device;
	cdsource_cdinfo *info;
	int x;
	
	tree=GTK_TREE(gtk_tree_new());
	
	for (x=0;x<cddrives_drives.drivecount;x++)
	  {		  		  
		  device=gtk_tree_item_new_with_label((char*)&cddrives_drives.drive[x]->device);
		  info=cdsource_cdinfo_create(cddrives_drives.drive[x]);
		  gtk_signal_connect(GTK_OBJECT(device),
				     "select",
				     selectcd,
				     info);
		  gtk_signal_connect(GTK_OBJECT(device),
				     "deselect",
				     unselectcd,
				     info);
		  gtk_tree_append(GTK_TREE(tree),device);
		  gtk_widget_show(device);
		  
	  };
	gtk_widget_show(GTK_WIDGET(tree));
	return tree;
}
;
	
	
	
	
