#include <gtk/gtk.h>

#include "main.h"
#include "colors.h"
#include "dircache.h"

#include "menubar.h"

#include "destcontrol.h"
#include "fsedit.h"
#include "trackedit.h"
#include "record.h"
#include "icons.h"

#include "source.h"

#include "preferences.h"

GtkWidget *window;
char *CRLF;

void delete_event(GtkWidget *widget,GdkEvent *event,gpointer data)
{
	gtk_main_quit();
}
;


void main_quit()
{
	trackedit_cleartracklist();
	gtk_main_quit();
}
;

void main(int argc,char **argv)
{	
	GtkWidget *surroundings;
	GtkWidget *mainscreen;
	
	GtkWidget *menubar;
	
	GtkWidget *destination; // a "Notebook" for the "destinations"
	GtkWidget *trackedit;	
	GtkWidget *fsedit;
	GtkWidget *record;

	GtkWidget *source;
	
	CRLF=(char*)malloc(3);
	CRLF[0]=13;
	CRLF[1]=10;
	CRLF[2]=0;
	
	gtk_init(&argc,&argv);
	
	/* load preferences before anything else to avoid complications... */
        preferences_init(argc,argv);
	
	dircache_loadcache("./dir.cache");
	
	window= gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window),"GnomeToaster 0.01");
	gtk_signal_connect(GTK_OBJECT(window),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL);
	gtk_widget_set_usize(GTK_WIDGET(window),INITSIZE_X,INITSIZE_Y);
	gtk_widget_show(window);
	
	colors_init();
	
	isotrack_init();
	audiotrack_init();
	cddrives_init(); /* do execute this before source_init() */
		
	surroundings=gtk_vbox_new(0,0);
	gtk_container_add(GTK_CONTAINER(window),surroundings);
	gtk_widget_show(surroundings);
	
	menubar=menubar_create();
	gtk_box_pack_start(GTK_BOX(surroundings),menubar,0,0,2);
	gtk_widget_show(menubar);
	
	mainscreen=gtk_vpaned_new();
	gtk_box_pack_start(GTK_BOX(surroundings),mainscreen,1,1,1);

	source=source_create();
       	gtk_paned_pack1(GTK_PANED(mainscreen),source,1,1);
	gtk_widget_show(source);
	
	destination=destination_create();

	trackedit=trackedit_create();
	gtk_widget_show(trackedit);

	fsedit=fsedit_create();
	gtk_widget_show(fsedit);
	
	record=record_create();
	gtk_widget_show(record);
	
	destination_add(GTK_NOTEBOOK(destination),fsedit,fsedit_xpm);
	destination_add(GTK_NOTEBOOK(destination),trackedit,trackedit_xpm);
	destination_add(GTK_NOTEBOOK(destination),record,record_xpm);
		
       	gtk_paned_pack2(GTK_PANED(mainscreen),destination,1,1);
	gtk_widget_show(destination);

	gtk_paned_compute_position(GTK_PANED(mainscreen),INITSIZE_Y,0,0);
	gtk_paned_set_position(GTK_PANED(mainscreen),2*(int)(INITSIZE_Y/3));	
	gtk_widget_show(mainscreen);

	/* those two have to be called *after* the widgets have been created
	 * because they set the default position of the selection cursor */
	source_init();
	fsedit_init();
	
	gtk_main();
	
	dircache_savecache("./dir.cache");
	preferences_exit();
}
;
	
