/* The Type of Track representing a raw cd track coming directly from cdrom */

#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "tracks.h"
#include "precachetrack.h"
#include "preferences.h"
#include "datacopydlg.h"

/* information handed to the several functions used via callback to 
 * preread a track */

typedef struct 
{
	tracks_trackinfo *dc;
	precachetrack_info *info;

	int readtrack;
	int writetrack;
	int readcount;
	char buf[BUFSIZE];
	
	void (*callback)(tracks_trackinfo*,int);
	int inputwatch;

	datacopydlg_dlginfo *dlg;
} precachetrack_copyinfo;
	

int precachetrack_openpipe(void*trackinfo)
{	
	precachetrack_info *i;
	
	i=(precachetrack_info*)((tracks_trackinfo*)trackinfo)->data;

	if (i->filedes==-1)
	  {
		  i->filedes=open((char*)&i->tempfile,O_RDONLY);
		  if (i->filedes==-1)
		      perror ("error opening data track");	
	  }
	;
	return i->filedes;
}
;

void precachetrack_closepipe(void*trackinfo)
{
	precachetrack_info *i;
	
	i=(precachetrack_info*)((tracks_trackinfo*)trackinfo)->data;
	if (i->filedes!=-1)
	  {		  
		  close(i->filedes);
		  i->filedes=-1;
	  }
	;
	
}
;

int precachetrack_tracksize(void*trackinfo)
{
	precachetrack_info *i;
	struct stat buf;
	
	i=(precachetrack_info*)((tracks_trackinfo*)trackinfo)->data;
	
	stat((char*)&i->tempfile,&buf);
		
	return buf.st_size;
}
;

int precachetrack_dataavail(void*trackinfo)
{
	precachetrack_info *i;

	i=(precachetrack_info*)((tracks_trackinfo*)trackinfo)->data;

	return (i->filedes!=-1);
}
;	

/* FIXME: add function tracks_invalidate() to tracks.h to regulate
 * the availability of the tracks located on a cdrom */
int  precachetrack_valid(void*trackinfo)  /* isotrack is always valid */
{
	return 1;
}
;

void precachetrack_destroy(void*trackinfo)
{
	precachetrack_info *i;
	
	i=(precachetrack_info*)((tracks_trackinfo*)trackinfo)->data;
	
	if (precachetrack_dataavail(trackinfo))
	    close (i->filedes);
	remove((char*)&i->tempfile);
	
	free(i);
}
;


void precachetrack_donecb(precachetrack_copyinfo *copy)
{
	gdk_input_remove(copy->inputwatch);
	
	tracks_closepipe(copy->info->client);
	close(copy->writetrack);
	
	datacopydlg_destroy(copy->dlg);
	
	copy->callback(copy->dc,1);
	free(copy);
}
;

void precachetrack_stopcb(GtkWidget *w,precachetrack_copyinfo *copy)
{
	gdk_input_remove(copy->inputwatch);
	
	tracks_closepipe(copy->info->client);
	close(copy->writetrack);

	datacopydlg_destroy(copy->dlg);
	
	/* degrade precache requirements of current track */
	if (copy->info->client->precacherequired>0)
	    copy->info->client->precacherequired--;

	/* free the incomplete precache track */
	tracks_unclaim(copy->dc);
	
	/* return client instead of precachetrack if track was not precached
	 * (do not unclaim client,as it should already have been unclaimed
	 * by the caller) */	    	    
	copy->callback(copy->info->client,0);
	free(copy);
	
}
;	

void precachetrack_newdatacb(gpointer data,gint source,
			     GdkInputCondition condition)
{
	precachetrack_copyinfo *copy;
	
	copy=(precachetrack_copyinfo*)data;
	
	copy->readcount=read(copy->readtrack,&copy->buf,BUFSIZE);
	if (copy->readcount>0)
	  {		  
		  write(copy->writetrack,&copy->buf,copy->readcount);
		  
		  datacopydlg_newdatablock(copy->dlg,copy->readcount);
	  }
	else
	    precachetrack_donecb(copy); 
}
;
	


tracks_trackinfo *precachetrack_create(tracks_trackinfo *client,
				       void (*callback)(tracks_trackinfo*,int)
				       )
{
	char copyname[1024];
	GtkWidget *stop;
	
	precachetrack_copyinfo *copyinfo;
	copyinfo=(precachetrack_copyinfo*)malloc(sizeof(precachetrack_copyinfo));
	
	copyinfo->info=(precachetrack_info*)malloc(sizeof(precachetrack_info));
	strcpy((char*)&copyinfo->info->tempfile,TEMPDIR);
	strcat((char*)&copyinfo->info->tempfile,"/");
	strcat((char*)&copyinfo->info->tempfile,(char*)&client->name);
	copyinfo->info->client=client;	
	copyinfo->info->filedes=-1;
	copyinfo->callback=callback;

	/* claim the client */
	tracks_claim(client);
	
	strcpy((char*)&copyname,(char*)&copyinfo->info->client->name);
	strcat((char*)&copyname," (precached)");	
	copyinfo->dc=tracks_create((char*)&copyname,
				   copyinfo->info->client->tracktype,none,
				   precachetrack_openpipe,
				   precachetrack_closepipe,
				   precachetrack_tracksize,
				   precachetrack_dataavail,
				   precachetrack_valid,
				   precachetrack_destroy,
				   copyinfo->info);

	copyinfo->dlg=datacopydlg_create("precaching file...",
					 tracks_tracksize(client),
					 GTK_SIGNAL_FUNC(precachetrack_stopcb),
					 copyinfo);
	
 	copyinfo->writetrack=open((char*)&copyinfo->info->tempfile,O_CREAT|O_RDWR);
	copyinfo->readtrack=tracks_openpipe(copyinfo->info->client);
	copyinfo->inputwatch=gdk_input_add(copyinfo->readtrack,
					   GDK_INPUT_READ,
					   precachetrack_newdatacb,
					   copyinfo);
	
	return copyinfo->dc;

}
;

	
