/* This is the "stream:/xxx" to tracks_trackinfo convert function
 * any valid stream has to be registered to that library with its stream:/xxx
 * name,so that it can be accessed from the track-editor and converted into
 * a normal Track-pipe, xxx is the corresponds to the stream name as stored
 * in tracks_trackinfo::name,so this identifier has to be unique */

#include "tracks.h"
#include "streams.h"
#include "main.h"

tracks_trackinfo *streams_stream[STREAMS_MAX];
int streams_count=0;

/* register a stream, return -1 if no more space and running number of the
 * stream,if successful. */
int streams_register(tracks_trackinfo *track)
{
	int result;
	
	if (streams_count<STREAMS_MAX)
	  {
		  tracks_claim(track);
		  streams_stream[streams_count]=track;		  
		  result=streams_count;
		  streams_count++;
		  #ifdef DEBUG
		      printf ("registered %s \n",(char*)&track->name);
		  #endif
	  }
	else
	  {
		  result=-1;
	  }
	;
	return result;
}
;

/* unregister a stream by its running number */
void streams_unregisterbynumber(int n)
{
	#ifdef DEBUG
	    printf ("unregistering %s \n",streams_stream[n]);
	#endif
	tracks_unclaim(streams_stream[n]);
	memcpy(&streams_stream[n],&streams_stream[n+1],streams_count-n-1);	
	streams_count--;
}
;


/* get the running number of a stream by specifying its name 
 * returns -1 if not found */
int streams_getstreamnumberbyname(char *name)
{
	int i;
	int result;
	
	result=-1;
	for (i=0;(i<streams_count)&&(result==-1);i++)
	  {
		  if (!strcmp((char*)&streams_stream[i]->name,name))
		    {
			    result=i;
		    }
		  ;
	  }
	;
	return result;
}
;

/* get a stream by its name,returns NULL if not found */
tracks_trackinfo *streams_getstreambyname(char *name)
{
	int sn;
	tracks_trackinfo *track;
	
	sn=streams_getstreamnumberbyname(name);
	if (sn!=-1)
	    track=streams_stream[sn];
	else
	    track=0;
	return track;
}
;

/* unregister a stream by its name */
void streams_unregisterbyname(char *name)
{
	int sn;
	
	sn=streams_getstreamnumberbyname(name);
	if (sn!=-1)
	    streams_unregisterbynumber(sn);
}
;

